#!/bin/bash
# script lancé par pam_exec pour faire une synchro locale des profils du serveur pour eviter les bugs de microcoupure reseau
# qu'on avait lorsque le home était monté en sshfs
# appellé par /etc/pam.d/common-session
#
# exemple a ajouter au fichier pam.d/common-session
#
#
# session	optional	pam_ldap.so 
# #a la fermture de session le script doit etre lance AVANT pam_mount qui demonte le repetoire
# session optional	pam_exec.so log=/tmp/pam.log /usr/local/bin/horizon-pam_session.sh  -o before_mount
# session	optional	pam_mount.so 
# #et a l'ouverture de session il faut le lancer APRES que pam_mount ait rendu le repertoire accessible
# session optional	pam_exec.so log=/tmp/pam.log /usr/local/bin/horizon-pam_session.sh -o after_mount
#
#
# pour les autres details cf http://redmine.abuledu.org/issues/1789
#
# Eric Seigne <eric.seigne@abuledu.org> 2017 / GNU/GPLv3
#
#

export LC_ALL=C

#pour connaitre le homedir de l'utilisateur ...
HZN_HOMEDIR=`getent passwd $PAM_USER | cut -d ":" -f6`
HZN_PROFILEDIRTEMP="/home/profiletemp"
HZN_PROGRESSFLAG="/var/tmp/horizon-pam_session_$PAM_USER.flag"

while [[ $# -gt 1 ]]
do
    key="$1"
    case $key in
	-o)
	    ABOUTPAMMOUNT="$2"
	    shift
	    ;;
	*)
	    ;;
    esac
    shift
done

#a l'ouverture de la session on ne sait pas si on est AVANT pam_mount ou apres ...
#alors on utilise la variable ABOUTPAMMOUNT pour savoir ...
if [ -z "$PAM_TYPE" -o "$PAM_TYPE" == "open_session" ]; then
    if [ "${ABOUTPAMMOUNT}" == "before_mount" ]; then
	exit 0
    fi

    echo "*******************************************************"
    echo "********************** LOGIN **************************"
    sleep 2
    #si pam_mount n'est pas encore passé on quitte
    TESTMNT=`mount | grep "${HZN_PROFILEDIRTEMP}/${PAM_USER}"`
    if [ -z "${TESTMNT}" ]; then
	echo " montage sshfs pas encore dispo pour $PAM_USER..."
	exit
    fi
    #si le repertoire n'existe pas
    if [ ! -d $HZN_HOMEDIR ]; then
	mkdir -p $HZN_HOMEDIR
    fi
    chown $PAM_USER $HZN_HOMEDIR

    # 1.espace disque saturé -> on pourrait essayer de supprimer les vieux profils locaux s'il y en a ...
    echo "  nettoyage des vieux profils locaux de plus de 28 jours ..."
    for localprofile in `find $HZN_PROFILEDIRTEMP/*/.horizon-last_session -atime +28`
    do
	localprofileparentdir=${localprofile%.horizon-last_session}
	echo -n "  -> ${localprofileparentdir} ..."
	if [ ! -z "${localprofileparentdir}" ]; then
	    #on s'assure que ce n'est pas le montage reseau ...
	    TESTMNTPARENT=`mount | grep "${localprofileparentdir}"`
	    if [ -z "${TESTMNTPARENT}" ]; then
		echo " est bien un espace local, suppression en cours"
		rm -rf ${localprofileparentdir}
	    else
		echo "erreur il semblerait que ça soit un montage réseau"
	    fi
	fi
    done

    #si le montage est dispo
    if [ -d $HZN_PROFILEDIRTEMP/$PAM_USER ]; then
	echo -n "  synchro en cours du serveur vers le disque local ... "
	echo "start" > $HZN_PROGRESSFLAG
	nohup /usr/local/bin/horizon-rsync_profile.sh -um mount -f $HZN_PROGRESSFLAG -u $PAM_USER -s $HZN_PROFILEDIRTEMP/$PAM_USER -d $HZN_HOMEDIR &
	#rsync -a $HZN_PROFILEDIRTEMP/$PAM_USER/.[^.]* $HZN_HOMEDIR/
	#que faire en cas d'erreur de synchro, par exemple espace disque saturé ?
	if [ "$?" == "0" ]; then
	    #a priori aucun pb de rsync -> on touch un fichier_flag
	    touch $HZN_HOMEDIR/.horizon-last_session
	    chown $PAM_USER $HZN_HOMEDIR/.horizon-last_session
	    echo " [ok]"
	else
	    #des pb de rsync ... par contre on ne sais pas lesquels, on va donc essayer de gerer les cas les plus
	    #classiques -> TODO
	    echo " [err]"
	fi
    fi
    echo "****************** END LOGIN **************************"
    echo "*******************************************************"
fi

#a la fermeture de la session on est toujours apres le umount et donc on n'a plus acces au repertoire de stockage
# -> solution de définir une commande spéciale dans pam_mount.xml pour le umount qui est en fait un script qui fait
# la synchro puis le umount réel
if [ "$PAM_TYPE" == "close_session" ]
then
    if [ "${ABOUTPAMMOUNT}" == "after_mount" ]; then
	exit 0
    fi
    echo "*******************************************************"
    echo "********************** LOGOUT *************************"
    #verifier que le repertoire du serveur est toujours monte
    TESTMNT=`mount | grep "${HZN_PROFILEDIRTEMP}/${PAM_USER}"`
    echo "résultat du test du montage ... $TESTMNT"
    if [ -z "${TESTMNT}" ]; then
	echo "  il semblerait que le montage soit démonté, on ne sauvegarde donc pas"
	echo "  les modifications locales ..."
    else
	echo "  il semblerait que le montage soit accessible ... sauvegarde en cours"
	# rsync -rlptoD $HZN_HOMEDIR/.[^.]* $HZN_PROFILEDIRTEMP/$PAM_USER/
	nohup /usr/local/bin/horizon-rsync_profile.sh -um umount -f $HZN_PROGRESSFLAG -u $PAM_USER -d $HZN_PROFILEDIRTEMP/$PAM_USER -s $HZN_HOMEDIR &
    fi
    echo "****************** END LOGOUT *************************"
    echo "*******************************************************"
fi

