#!/bin/bash
#script special pour synchoniser des profils entre deux repertoires
#l'un des deux pouvant être un point de montage ...
#pratique pour faire une synchro entre client et serveur
HZN_RSYNCEXCLUDEFILE="/usr/share/horizon/horizon-pam_session.exclude"

if [ -f /usr/share/horizon/horizon-pam_session.exclude.local ]; then
    HZN_RSYNCEXCLUDEFILE="/usr/share/horizon/horizon-pam_session.exclude.local"
fi

while [[ $# -gt 1 ]]
do
    key="$1"
    case $key in
	-f)
	    FLAGFILE="$2"
	    shift
	    ;;
	-u)
	    PAM_USER="$2"
	    shift
	    ;;
	-s)
	    SRCDIR="$2"
	    shift
	    ;;
	-d)
	    DSTDIR="$2"
	    shift
	    ;;
	-um)
	    UMOUNT="$2"
	    shift
	    ;;
	*)
	    ;;
    esac
    shift
done

touch ${FLAGFILE}

#verifier a l'ouverture de session que le repertoire source est bien un montage reseau
TESTMNT=""
if [ "${UMOUNT}" != "umount" ]; then
    TESTMNT=`mount | grep "${SRCDIR}"`
else
    #et a la fermture de session idem
    TESTMNT=`mount | grep "${DSTDIR}"`
fi
if [ -z "${TESTMNT}" ]; then
    echo " montage sshfs absent ... exit"
    exit -1
else

    echo " D: on lance rsync --exclude-from=${HZN_RSYNCEXCLUDEFILE} -rlptoD ${SRCDIR}/.[^.]* ${DSTDIR}/"
    rsync --exclude-from=${HZN_RSYNCEXCLUDEFILE} -rlptoD ${SRCDIR}/.[^.]* ${DSTDIR}/

    if [ "$?" == "0" ]; then
	#a priori aucun pb de rsync -> on touch un fichier_flag
	touch $DSTDIR/.horizon-last_session
	chown $PAM_USER $DSTDIR/.horizon-last_session
	echo " [ok]"
	
	#creation des liens symboliques pour l'accès aux documents uniquement si on est en ouverture de session
	#donc si on est en phase ou il faut faire le umount ... pas de symlink !
	if [ "${UMOUNT}" != "umount" ]; then
	    cd $DSTDIR
	    if [ ! -L "Serveur_de_fichiers" -a ! -e "Serveur_de_fichiers" ]; then
     		ln -s "$SRCDIR" "Serveur_de_fichiers"
     		echo " - Serveur_de_fichiers -> $DSTDIR"
	    fi
	    for ENTRY in Bureau Documents Téléchargements Images Musique leterrier Sankore "Sankore 3.1" .thunderbird
	    do
    		(
    		    cd $DSTDIR
    		    if [ ! -L "${ENTRY}" -a ! -e "${ENTRY}" -a -e "$SRCDIR/${ENTRY}" ]; then
    			ln -s "$SRCDIR/${ENTRY}" "$DSTDIR/${ENTRY}"
    			echo " - $DSTDIR/${ENTRY} -> $SRCDIR/${ENTRY}"
    		    fi
    		)
	    done
	fi
	
	#bug / cas particulier du .ssh/known_hosts qui a peut etre l'identité d'un autre serveur abuledu et donc il
	#faut le supprimer
	for serveur in 192.168.0.1 servecole
	do
	    ssh-keygen -f "$SRCDIR/.ssh/known_hosts" -R ${serveur}
	    ssh-keygen -f "$DSTDIR/.ssh/known_hosts" -R ${serveur}
	done

    else
	true
	#des pb de rsync ... par contre on ne sais pas lesquels, on va donc essayer de gerer les cas les plus
	#classiques -> TODO
	echo " [err]"
    fi
    
    if [ "${UMOUNT}" == "umount" ]; then
	#mais en phase de umount il faut absolument propager les suppressions "locales" vers le serveur ...
	#c'est meme critique pour des appli comme firefox ...
	for ENTRY in .config .gconf .mozilla .pki .firefox .kde .libreoffice .ssh
	do
	    echo "on supprime les fichiers pour ${ENTRY} ... ${SRCDIR}/${ENTRY}/ -> ${DSTDIR}/${ENTRY}/"
	    if [ -d "${SRCDIR}/${ENTRY}" -a -d "${DSTDIR}/${ENTRY}" ]; then
		rsync --delete -rvlptoD "${SRCDIR}/${ENTRY}/" "${DSTDIR}/${ENTRY}/"
	    fi
	done

	umount ${DSTDIR}
	#et on verifie si possible que le umount s'est bien fait ...
	sleep 2
	TESTMNT=""
	if [ "${UMOUNT}" != "umount" ]; then
	    TESTMNT=`mount | grep "${SRCDIR}"`
	else
	    #et a la fermture de session idem
	    TESTMNT=`mount | grep "${DSTDIR}"`
	fi
	if [ ! -z "${TESTMNT}" ]; then
	    umount ${DSTDIR}
	fi
    fi

    if [ -f ${FLAGFILE} ]; then
	echo "on efface ${FLAGFILE} ..."
	rm ${FLAGFILE}
    fi
fi
